@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2014-2020 Christian Schoenebeck
+    Copyright (c) 2014-2025 Christian Schoenebeck
 
     This file is part of "gigedit" and released under the terms of the
     GNU General Public License version 2.
@@ -26,6 +26,10 @@
 #include <gtkmm/messagedialog.h>
 #include <gtkmm/label.h>
 #include <gtk/gtkwidget.h> // for gtk_widget_modify_*()
+#if GTKMM_MAJOR_VERSION > 3 || (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION >= 20)
+# include <gtkmm/cssprovider.h>
+# include <gtkmm/stylecontext.h>
+#endif
 
 Glib::ustring dimTypeAsString(gig::dimension_t d);
 
@@ -101,7 +105,7 @@ static int findLowestRegionPoint(std::vector<gig::Instrument*>& instruments, int
     int result = -1;
     for (uint i = 0; i < instruments.size(); ++i) {
         gig::Instrument* instr = instruments[i];
-        for (gig::Region* rgn = instr->GetFirstRegion(); rgn; rgn = instr->GetNextRegion()) {
+        for (size_t r = 0; gig::Region* rgn = instr->GetRegionAt(r); ++r) {
             if (rgn->KeyRange.overlaps(searchRange)) {
                 int lowest = smallestOverlapPoint(rgn->KeyRange, searchRange);
                 if (result == -1 || lowest < result) result = lowest;
@@ -123,7 +127,7 @@ static int findFirstRegionEnd(std::vector<gig::Instrument*>& instruments, int iS
     int result = -1;
     for (uint i = 0; i < instruments.size(); ++i) {
         gig::Instrument* instr = instruments[i];
-        for (gig::Region* rgn = instr->GetFirstRegion(); rgn; rgn = instr->GetNextRegion()) {
+        for (size_t r = 0; gig::Region* rgn = instr->GetRegionAt(r); ++r) {
             if (rgn->KeyRange.overlaps(searchRange)) {
                 if (result == -1 || rgn->KeyRange.high < result)
                     result = rgn->KeyRange.high;
@@ -140,7 +144,7 @@ static int findFirstRegionEnd(std::vector<gig::Instrument*>& instruments, int iS
 static std::vector<gig::Region*> getAllRegionsWhichOverlapRange(gig::Instrument* instrument, DLS::range_t range) {
     //std::cout << "All regions which overlap { " << (int)range.low << ".." << (int)range.high << " } : " << std::flush;
     std::vector<gig::Region*> v;
-    for (gig::Region* rgn = instrument->GetFirstRegion(); rgn; rgn = instrument->GetNextRegion()) {
+    for (size_t r = 0; gig::Region* rgn = instrument->GetRegionAt(r); ++r) {
         if (rgn->KeyRange.overlaps(range)) {
             v.push_back(rgn);
             //std::cout << (int)rgn->KeyRange.low << ".." << (int)rgn->KeyRange.high << ", " << std::flush;
@@ -240,8 +244,10 @@ static Dimensions getDimensionsForRegionGroup(RegionGroup& regionGroup) {
             printf("\t[rgn=%p,dim=%#x] = {", rgn, def.dimension);
             #endif
             for (uint z = 0; z < def.zones; ++z) {
+/*
                 int dr = z << previousBits;
                 gig::DimensionRegion* dimRgn = rgn->pDimensionRegions[dr];
+ */
                 // NOTE: Originally this function collected dimensions' upper
                 // limits. However that caused combined instruments (e.g. with
                 // unequal dimension zone counts, not being a power of two) to
@@ -857,6 +863,8 @@ CombineInstrumentsDialog::CombineInstrumentsDialog(Gtk::Window& parent, gig::Fil
         set_position(Gtk::WIN_POS_MOUSE);
     }
 
+    const bool darkMode = isDarkMode();
+
     m_scrolledWindow.add(m_treeView);
     m_scrolledWindow.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
 
@@ -927,21 +935,18 @@ CombineInstrumentsDialog::CombineInstrumentsDialog(Gtk::Window& parent, gig::Fil
     );
     m_treeView.show();
 
-    for (int i = 0; true; ++i) {
-        gig::Instrument* instr = gig->GetInstrument(i);
-        if (!instr) break;
-
+    for (size_t i = 0; gig::Instrument* instr = gig->GetInstrument(i); ++i) {
 #if DEBUG_COMBINE_INSTRUMENTS
         {
             std::cout << "Instrument (" << i << ") '" << instr->pInfo->Name << "' Regions: " << std::flush;
-            for (gig::Region* rgn = instr->GetFirstRegion(); rgn; rgn = instr->GetNextRegion()) {
+            for (size_t r = 0; gig::Region* rgn = instr->GetRegionAt(r); ++r) {
                 std::cout << rgn->KeyRange.low << ".." << rgn->KeyRange.high << ", " << std::flush;
             }
             std::cout << std::endl;
         }
         std::cout << std::endl;
 #endif
-        
+
         Glib::ustring name(gig_to_utf8(instr->pInfo->Name));
         Gtk::TreeModel::iterator iter = m_refTreeModel->append();
         Gtk::TreeModel::Row row = *iter;
@@ -958,21 +963,31 @@ CombineInstrumentsDialog::CombineInstrumentsDialog(Gtk::Window& parent, gig::Fil
     // force background to retain white also on selections
     // (this also fixes a bug with GTK 2 which often causes visibility issue
     //  with the text of the selected item)
+#if GTK_MAJOR_VERSION < 3
     {
 #if (GTKMM_MAJOR_VERSION == 2 && GTKMM_MINOR_VERSION < 90) || GTKMM_MAJOR_VERSION < 2
         Gdk::Color white;
 #else
         Gdk::RGBA white;
 #endif
-        white.set("#ffffff");
+        white.set(darkMode ? "#202728" : "#ffffff");
         GtkWidget* widget = (GtkWidget*) m_iconView.gobj();
-#if GTK_MAJOR_VERSION < 3
         gtk_widget_modify_base(widget, GTK_STATE_SELECTED, white.gobj());
         gtk_widget_modify_base(widget, GTK_STATE_ACTIVE, white.gobj());
         gtk_widget_modify_bg(widget, GTK_STATE_SELECTED, white.gobj());
         gtk_widget_modify_bg(widget, GTK_STATE_ACTIVE, white.gobj());
-#endif
     }
+#endif
+#if GTKMM_MAJOR_VERSION > 3 || (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION >= 20)
+    {
+        auto css = Gtk::CssProvider::create();
+        Glib::ustring bgColor = darkMode ? "#202728" : "#ffffff";
+        css->load_from_data(
+            "* { background-color: " + bgColor + "; }"
+        );
+        m_iconView.get_style_context()->add_provider(css, GTK_STYLE_PROVIDER_PRIORITY_USER);
+    }
+#endif
 
     m_labelOrder.set_text(_("Order of the instruments to be combined:"));
 
@@ -1083,6 +1098,18 @@ void CombineInstrumentsDialog::on_order_drag_data_get(const Glib::RefPtr<Gdk::Dr
                        sizeof(src)/*length of data in bytes*/);
 }
 
+static Glib::ustring markupInstrumentIcon(int nr, Glib::ustring name, bool darkMode) {
+    const Glib::ustring fgColorNr = darkMode ? "#f2be00" : "black";
+    const Glib::ustring fgColorName = darkMode ? "#07c0cf" : "green";
+
+    return Glib::ustring::compose(
+        "<span foreground='%2'>%1.</span>\n"
+        "<span foreground='%4'>%3</span>",
+        nr, fgColorNr,
+        name, fgColorName
+    );
+}
+
 void CombineInstrumentsDialog::on_order_drop_drag_data_received(
     const Glib::RefPtr<Gdk::DragContext>& context, int x, int y,
     const Gtk::SelectionData& selection_data, guint, guint time)
@@ -1146,16 +1173,15 @@ void CombineInstrumentsDialog::on_order_drop_drag_data_received(
         // update markup
         Gtk::TreeModel::Row rowSrc = *itSrc;
         Gtk::TreeModel::Row rowDst = *itDst;
+        const bool darkMode = isDarkMode();
         {
             Glib::ustring name = rowSrc[m_orderColumns.m_col_name];
-            Glib::ustring markup =
-                "<span foreground='black' background='white'>" + ToString(iDst+1) + ".</span>\n<span foreground='green' background='white'>" + name + "</span>";
+            Glib::ustring markup = markupInstrumentIcon(iDst+1, name, darkMode);
             rowSrc[m_orderColumns.m_col_markup] = markup;
         }
         {
             Glib::ustring name = rowDst[m_orderColumns.m_col_name];
-            Glib::ustring markup =
-                "<span foreground='black' background='white'>" + ToString(iSrc+1) + ".</span>\n<span foreground='green' background='white'>" + name + "</span>";
+            Glib::ustring markup = markupInstrumentIcon(iSrc+1, name, darkMode);
             rowDst[m_orderColumns.m_col_markup] = markup;
         }
     }
@@ -1305,14 +1331,14 @@ void CombineInstrumentsDialog::onSelectionChanged() {
     // update markup
     {
         int i = 0;
+        const bool darkMode = isDarkMode();
         Children allOrdered = m_refOrderModel->children();
         for (Children::iterator itOrder = allOrdered.begin();
              itOrder != allOrdered.end(); ++itOrder, ++i)
         {
             Gtk::TreeModel::Row rowOrder = *itOrder;
             Glib::ustring name = rowOrder[m_orderColumns.m_col_name];
-            Glib::ustring markup =
-                "<span foreground='black' background='white'>" + ToString(i+1) + ".</span>\n<span foreground='green' background='white'>" + name + "</span>";
+            Glib::ustring markup = markupInstrumentIcon(i+1, name, darkMode);
             rowOrder[m_orderColumns.m_col_markup] = markup;
         }
     }
