@@ -1,5 +1,6 @@
 /*                                                         -*- c++ -*-
  * Copyright (C) 2011-2019 Andreas Persson
+ * Copyright (C) 2017-2025 Christian Schoenebeck
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -498,4 +499,12 @@ private:
     Glib::ustring m_markup;
 };
 
+#if GTKMM_MAJOR_VERSION >= 3
+bool isDarkMode();
+#else
+inline bool isDarkMode() {
+    return false;
+}
+#endif
+
 #endif // GIGEDIT_COMPAT_H
