@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2006-2019 Andreas Persson
+ * Copyright (C) 2007-2025 Christian Schoenebeck
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -368,7 +369,7 @@ int note_value(const Glib::ustring& note)
 Glib::ustring note_str(int note)
 {
     char buf[10];
-    sprintf(buf, "%s%d", notes[note % 12], note / 12 - 1);
+    snprintf(buf, sizeof(buf), "%s%d", notes[note % 12], note / 12 - 1);
     return buf;
 }
 
