@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2014-2020 Christian Schoenebeck
+    Copyright (c) 2014-2025 Christian Schoenebeck
     
     This file is part of "gigedit" and released under the terms of the
     GNU General Public License version 2.
@@ -98,11 +98,17 @@ ScriptEditor::ScriptEditor() :
     errorIconNames.push_back("process-stop");
 
     std::vector<std::string> warningIconNames;
-    warningIconNames.push_back("dialog-warning-symbolic");
     warningIconNames.push_back("dialog-warning");
+    warningIconNames.push_back("dialog-warning-symbolic");
 
     std::vector<std::string> successIconNames;
+    successIconNames.push_back("checkmark");
+    successIconNames.push_back("emblem-checked");
+    successIconNames.push_back("dialog-positive");
+    successIconNames.push_back("emblem-ok");
     successIconNames.push_back("emblem-default");
+    successIconNames.push_back("dialog-ok");
+    successIconNames.push_back("dialog-ok-apply");
     successIconNames.push_back("tools-check-spelling");
 
     m_errorIcon = createIcon(errorIconNames, get_screen());
@@ -113,39 +119,53 @@ ScriptEditor::ScriptEditor() :
 
     m_tagTable = Gtk::TextBuffer::TagTable::create();
 
+    const bool darkMode = isDarkMode();
+
+    const Glib::ustring black = darkMode ? "#d2d2d2" : "#000000";
+    const Glib::ustring cyan = "#07c0cf";
+    const Glib::ustring darkCyan = darkMode ? "#668cff" : "#1ba1dd";
+    const Glib::ustring magenta = darkMode ? "#ae39ff" : "#790cc4";
+    const Glib::ustring bronze = darkMode ? "#f2be00" : "#c4950c";
+    const Glib::ustring red = darkMode ? "#af0f0f" : "#ff9393";
+    const Glib::ustring darkRed = darkMode ? "#ff3461" : "#c40c0c";
+    const Glib::ustring pink = "#FF4FF3";
+    const Glib::ustring gray = "#9c9c9c";
+    const Glib::ustring yellow = darkMode ? "#5b5e00" : "#fffd7c";
+    const Glib::ustring liteGreen = "#50BC00";
+
     m_keywordTag = Gtk::TextBuffer::Tag::create();
-    m_keywordTag->property_foreground() = "#000000"; // black
+    m_keywordTag->property_foreground() = black;
     m_keywordTag->property_weight() = PANGO_WEIGHT_BOLD;
     m_tagTable->add(m_keywordTag);
 
     m_eventTag = Gtk::TextBuffer::Tag::create();
-    m_eventTag->property_foreground() = "#07c0cf"; // cyan 1
+    m_eventTag->property_foreground() = cyan;
     m_eventTag->property_weight() = PANGO_WEIGHT_BOLD;
     m_tagTable->add(m_eventTag);
     
     m_variableTag = Gtk::TextBuffer::Tag::create();
-    m_variableTag->property_foreground() = "#790cc4"; // magenta
+    m_variableTag->property_foreground() = magenta;
     m_tagTable->add(m_variableTag);
     
     m_functionTag = Gtk::TextBuffer::Tag::create();
-    m_functionTag->property_foreground() = "#1ba1dd"; // cyan 2
+    m_functionTag->property_foreground() = darkCyan;
     m_tagTable->add(m_functionTag);
     
     m_numberTag = Gtk::TextBuffer::Tag::create();
-    m_numberTag->property_foreground() = "#c4950c"; // yellow
+    m_numberTag->property_foreground() = bronze;
     m_tagTable->add(m_numberTag);
 
     m_stringTag = Gtk::TextBuffer::Tag::create();
-    m_stringTag->property_foreground() = "#c40c0c"; // red
+    m_stringTag->property_foreground() = darkRed;
     m_tagTable->add(m_stringTag);
 
     m_patchTag = Gtk::TextBuffer::Tag::create();
-    m_patchTag->property_foreground() = "#FF4FF3"; // pink
+    m_patchTag->property_foreground() = pink;
     m_patchTag->property_weight() = PANGO_WEIGHT_BOLD;
     m_tagTable->add(m_patchTag);
 
     m_commentTag = Gtk::TextBuffer::Tag::create();
-    m_commentTag->property_foreground() = "#9c9c9c"; // gray
+    m_commentTag->property_foreground() = gray;
     m_tagTable->add(m_commentTag);
 
     #define PREPROC_TOKEN_COLOR  "#2f8a33" // green
@@ -156,27 +176,27 @@ ScriptEditor::ScriptEditor() :
 
     m_preprocCommentTag = Gtk::TextBuffer::Tag::create();
     m_preprocCommentTag->property_strikethrough() = true;
-    m_preprocCommentTag->property_background() = "#e5e5e5";
+    m_preprocCommentTag->property_background() = darkMode ? "#2d3d33" : "#e5e5e5";
     m_tagTable->add(m_preprocCommentTag);
 
     m_errorTag = Gtk::TextBuffer::Tag::create();
-    m_errorTag->property_background() = "#ff9393"; // red
+    m_errorTag->property_background() = red;
     m_tagTable->add(m_errorTag);
 
     m_warningTag = Gtk::TextBuffer::Tag::create();
-    m_warningTag->property_background() = "#fffd7c"; // yellow
+    m_warningTag->property_background() = yellow;
     m_tagTable->add(m_warningTag);
 
     m_lineNrTag = Gtk::TextBuffer::Tag::create();
-    m_lineNrTag->property_foreground() = "#CCCCCC";
+    m_lineNrTag->property_foreground() = darkMode ? "#7e7e7e" : "#CCCCCC";
     m_tagTable->add(m_lineNrTag);
 
     m_metricTag = Gtk::TextBuffer::Tag::create();
-    m_metricTag->property_foreground() = "#000000"; // black
+    m_metricTag->property_foreground() = black;
     m_tagTable->add(m_metricTag);
 
     m_stdUnitTag = Gtk::TextBuffer::Tag::create();
-    m_stdUnitTag->property_foreground() = "#50BC00"; // greenish
+    m_stdUnitTag->property_foreground() = liteGreen;
     m_tagTable->add(m_stdUnitTag);
 
     // create menu
@@ -270,13 +290,22 @@ ScriptEditor::ScriptEditor() :
     m_lineNrTextViewSpacer.set_size_request(5,14);
     m_lineNrTextViewSpacer.property_editable() = false;
     m_lineNrTextViewSpacer.property_sensitive() = false;
+    {
+        Gdk::Color color;
+        color.set(darkMode ? "#202728" : "#ffffff");
+        GtkWidget* widget = (GtkWidget*) m_textView.gobj();
+        #if GTK_MAJOR_VERSION < 3 || (GTK_MAJOR_VERSION == 3 && GTK_MINOR_VERSION <= 24)
+        gtk_widget_modify_base(widget, GTK_STATE_NORMAL, color.gobj());
+        gtk_widget_modify_bg(widget, GTK_STATE_NORMAL, color.gobj());
+        #endif
+    }
     {
 #if 1 //(GTKMM_MAJOR_VERSION == 2 && GTKMM_MINOR_VERSION < 90) || GTKMM_MAJOR_VERSION < 2
         Gdk::Color color;
 #else
         Gdk::RGBA color;
 #endif
-        color.set("#F5F5F5");
+        color.set(darkMode ? "#212121" : "#F5F5F5");
         GtkWidget* widget = (GtkWidget*) m_lineNrView.gobj();
 #if GTK_MAJOR_VERSION < 3 || (GTK_MAJOR_VERSION == 3 && GTK_MINOR_VERSION <= 24)
         gtk_widget_modify_base(widget, GTK_STATE_NORMAL, color.gobj());
@@ -289,7 +318,7 @@ ScriptEditor::ScriptEditor() :
 #else
         Gdk::RGBA color;
 #endif
-        color.set("#EEEEEE");
+        color.set(darkMode ? "#272727" : "#EEEEEE");
         GtkWidget* widget = (GtkWidget*) m_lineNrTextViewSpacer.gobj();
 #if GTK_MAJOR_VERSION < 3 || (GTK_MAJOR_VERSION == 3 && GTK_MINOR_VERSION <= 24)
         gtk_widget_modify_base(widget, GTK_STATE_NORMAL, color.gobj());
@@ -410,13 +439,6 @@ int ScriptEditor::currentFontSize() const {
 
 void ScriptEditor::setFontSize(int sizePt, bool save) {
     //printf("setFontSize(%d,%d)\n", size, save);
-
-    // make sure the real size on the screen for the editor's font is consistent
-    // on all screens (which otherwise may vary between models and DPI settings)
-    const double referenceDPI = 96;
-    double dpi = Gdk::Screen::get_default()->get_resolution();
-    double sizePx = sizePt * dpi / referenceDPI;
-
 #if GTKMM_MAJOR_VERSION < 3 || (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION < 20)
     Pango::FontDescription fdesc;
     fdesc.set_family("monospace");
@@ -425,6 +447,11 @@ void ScriptEditor::setFontSize(int sizePt, bool save) {
     if (macIsMinMac10_6())
         fdesc.set_family("Menlo");
 # endif
+    // make sure the real size on the screen for the editor's font is consistent
+    // on all screens (which otherwise may vary between models and DPI settings)
+    const double referenceDPI = 96;
+    const double dpi = Gdk::Screen::get_default()->get_resolution();
+    const double sizePx = sizePt * dpi / referenceDPI;
     fdesc.set_size(sizePx * PANGO_SCALE);
 # if GTKMM_MAJOR_VERSION < 3
     m_lineNrView.modify_font(fdesc);
@@ -442,14 +469,22 @@ void ScriptEditor::setFontSize(int sizePt, bool save) {
 # endif
     if (!m_css) {
         m_css = Gtk::CssProvider::create();
-        m_lineNrView.get_style_context()->add_provider(m_css, GTK_STYLE_PROVIDER_PRIORITY_FALLBACK);
-        m_textView.get_style_context()->add_provider(m_css, GTK_STYLE_PROVIDER_PRIORITY_FALLBACK);
+        m_lineNrView.get_style_context()->add_provider(m_css, GTK_STYLE_PROVIDER_PRIORITY_USER);
+        m_textView.get_style_context()->add_provider(m_css, GTK_STYLE_PROVIDER_PRIORITY_USER);
     }
-    m_css->load_from_data(
+    Glib::ustring css =
         "* {"
         "  font: " + ToString(sizePt) + "pt " + family + ";"
-        "}"
-    );
+        "}";
+    const bool darkMode = isDarkMode();
+    if (darkMode) {
+        css +=
+            "\n"
+            "* selection {"
+            "  background-color: #59606d;"
+            "}";
+    }
+    m_css->load_from_data(css);
 #endif
     if (save) Settings::singleton()->scriptEditorFontSize = sizePt;
 }
@@ -726,7 +761,10 @@ static std::string warningsCountTxt(const std::vector<LinuxSampler::ParserIssue>
 }
 
 static std::string errorsCountTxt(const std::vector<LinuxSampler::ParserIssue> errors) {
-    std::string txt = "<span foreground=\"#c40c0c\">" + ToString(errors.size());
+    const bool darkMode = isDarkMode();
+    std::string txt =
+        (darkMode ? "<span foreground=\"#ff1f1f\">" : "<span foreground=\"#c40c0c\">") +
+        ToString(errors.size());
     txt += (errors.size() == 1) ? " Error" : " Errors";
     txt += "</span>";
     return txt;
